from micropm4py.log import xes_import_traces, xes_export_traces, xes_import_traces_file, xes_export_traces_file, csv_import_traces, csv_import_traces_file, csv_export_traces, csv_export_traces_file, xes_import_traces_file_standard, postprocessing



from micropm4py.log import csv_import_traces


def import_traces_path(file_path, sep, ci, ai):
    F = open(file_path, "r")
    line = F.readline()
    cidp, acp = csv_import_traces.import_header(line, sep, ci, ai)
    d = {}
    da = {}
    line = F.readline()
    while line:
        d = csv_import_traces.import_traces(d, line, sep, cidp, acp, da)
        line = F.readline()
    F.close()
    d = csv_import_traces.finish_traces(d)
    return d


def import_dfg_path(file_path, sep, ci, ai):
    F = open(file_path, "r")
    line = F.readline()
    cidp, acp = csv_import_traces.import_header(line, sep, ci, ai)
    dfg = [[], dict(), set(), set(), dict()]
    d = {}
    line = F.readline()
    while line:
        dfg, d = csv_import_traces.import_dfg(dfg, d, line, sep, cidp, acp)
        line = F.readline()
    F.close()
    dfg = csv_import_traces.finish_dfg(dfg, d)
    return dfg


def import_dfg_path_sten(file_path, sep, ci, ai):
    F = open(file_path, "r")
    line = F.readline()
    cidp, acp = csv_import_traces.import_header(line, sep, ci, ai)
    dfg = [[">>", "[]"], {0: 0, 1: 0}, {0}, {1}, dict()]
    d = {}
    line = F.readline()
    while line:
        dfg, d = csv_import_traces.import_dfg_sten(dfg, d, line, sep, cidp, acp)
        line = F.readline()
    F.close()
    dfg = csv_import_traces.finish_dfg_sten(dfg, d)
    return dfg



def save_dot(file_path, net):
    F = open(file_path, "w")
    F.write("digraph graphname {\n")
    i = 0
    while i < len(net[0]):
        F.write("p%d [label=\"\"];\n" % (i,))
        i = i + 1
    i = 0
    while i < len(net[1]):
        F.write("t%d [label=\"%s\", shape=box];\n" % (i, net[1][i][0]))
        i = i + 1
    i = 0
    while i < len(net[1]):
        for p in net[1][i][1]:
            F.write("p%d -> t%d;\n" % (p, i))
        for p in net[1][i][2]:
            F.write("t%d -> p%d;\n" % (i, p))
        i = i + 1
    F.write("}\n")
    F.close()



import time

# RAN ON CORTEX M3, 64kb RAM
def tr_to_dfg0(tr, dfg):
    if len(tr) > 0:
        i = 0
        while i < len(tr):
            if tr[i] not in dfg[0]:
                dfg[0].append(tr[i])
                dfg[1][len(dfg[0]) - 1] = 0
            dfg[1][dfg[0].index(tr[i])] = dfg[1][dfg[0].index(tr[i])] + 1
            if i > 0:
                tup = (dfg[0].index(tr[i - 1]), dfg[0].index(tr[i]))
                if not tup in dfg[4]:
                    dfg[4][tup] = 0
                dfg[4][tup] = dfg[4][tup] + 1
            i = i + 1
        dfg[2].add(dfg[0].index(tr[0]))
        dfg[3].add(dfg[0].index(tr[-1]))
    return dfg


def tr_to_dfg(tr, dfg):
    if len(tr) == 2 and type(tr[1]) is tuple:
        return tr_to_dfg0(tr[1], dfg)
    return tr_to_dfg0(tr, dfg)


def trs_to_dfg(trs):
    dfg = [[], dict(), set(), set(), dict()]
    for tr in trs:
        dfg = tr_to_dfg(tr, dfg)
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    return dfg


def main():
    dfg = [[], dict(), set(), set(), dict()]
    dfg = tr_to_dfg(("A", "B", "C"), dfg)
    dfg = tr_to_dfg(("A", "B"), dfg)
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    print(dfg)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



import time

# RAN ON CORTEX M3, 64kb RAM


def prnt_dot(dfg):
    print("digraph graphname {")
    i = 0
    while i < len(dfg[0]):
        print("%d [label=\"%s (%d)\"];" % (i, dfg[0][i], dfg[1][i]))
        i = i + 1
    for tup in dfg[4]:
        print("%d -> %d [label=\"%d\"];" % (tup[0], tup[1], dfg[4][tup]))
    print("}")


def main():
    dfg = [("A", "B", "C"), {0: 2, 1: 2, 2: 1}, tuple({0}), tuple({1, 2}), {(0, 1): 2, (1, 2): 1}]
    prnt_dot(dfg)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



import time

# RAN ON CORTEX M3, 64kb RAM
def exec(p, m, a):
    for t in p[1]:
        if t[0] == a:
            o = True
            for p, v in t[1].items():
                if p not in m or v < m[p]:
                    o = False
                    break
            if o:
                for p, v in t[1].items():
                    m[p] = m[p] - v
                    if m[p] == 0:
                        del m[p]
                for p, v in t[2].items():
                    if p not in m:
                        m[p] = 0
                    m[p] = m[p] + v
                return m


def ex_trace(p, m, fm, tr):
    for act in tr:
        m = exec(p, m, act)
        if m is None:
            return False
    return m == fm


def main():
    p = [("source", "p1", "sink"), (("A", {0: 1}, {1: 1}), ("B", {1: 1}, {2: 1}))]
    r = ex_trace(p, {0: 1}, {2: 1}, ("A", "B"))
    print(r)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



def to(dfg0):
    acti_count = {}
    dfg = {}
    start_activities = set()
    end_activities = set()

    for act in dfg0[1]:
        acti_count[dfg0[0][act]] = dfg0[1][act]

    for tup in dfg0[4]:
        new_tup = (dfg0[0][tup[0]], dfg0[0][tup[1]])
        dfg[new_tup] = dfg0[4][tup]

    for act in dfg0[2]:
        start_activities.add(dfg0[0][act])

    for act in dfg0[3]:
        end_activities.add(dfg0[0][act])

    return dfg, acti_count, start_activities, end_activities



from pm4py.objects.petri.petrinet import PetriNet, Marking
from pm4py.objects.petri.utils import petri as petri_utils


def to(net0, im0, fm0):
    net = PetriNet("")
    im = Marking()
    fm = Marking()
    dp = {}
    dt = {}
    i = 0
    while i < len(net0[0]):
        p = PetriNet.Place(net0[0][i])
        net.places.add(p)
        dp[i] = p
        i = i + 1
    i = 0
    while i < len(net0[1]):
        t = PetriNet.Transition("t%d" % (i), net0[1][i][0])
        net.transitions.add(t)
        dt[i] = t
        i = i + 1
    i = 0
    while i < len(net0[1]):
        for p in net0[1][i][1]:
            petri_utils.utils.add_arc_from_to(dp[p], dt[i], net, weight=net0[1][i][1][p])
        for p in net0[1][i][2]:
            petri_utils.utils.add_arc_from_to(dt[i], dp[p], net, weight=net0[1][i][2][p])
        i = i + 1
    for p in im0:
        im[dp[p]] = im0[p]
    for p in fm0:
        fm[dp[p]] = fm0[p]
    return net, im, fm


def frm(net, im, fm):
    net0 = [[], []]
    im0 = {}
    fm0 = {}
    places = list(net.places)
    transitions = list(net.transitions)
    dp = {places[i]:i for i in range(len(places))}
    for p in places:
        net0[0].append(p.name)
    for t in transitions:
        net0[1].append([t.label, {}, {}])
        for a in t.in_arcs:
            p = a.source
            w = a.weight
            net0[1][-1][1][dp[p]] = w
        for a in t.out_arcs:
            p = a.target
            w = a.weight
            net0[1][-1][2][dp[p]] = w
    for p in im:
        im0[dp[p]] = im[p]
    for p in fm:
        fm0[dp[p]] = fm[p]
    net0[0] = tuple(net0[0])
    i = 0
    while i < len(net0[1]):
        net0[1][i] = tuple(net0[1][i])
        i = i + 1
    net0[1] = tuple(net0[1])
    return net0, im0, fm0



import time

# RAN ON CORTEX M3, 64kb RAM


def print_dot(net):
    print("digraph graphname {")
    i = 0
    while i < len(net[0]):
        print("p%d [label=\"\"];" % (i,))
        i = i + 1
    i = 0
    while i < len(net[1]):
        print("t%d [label=\"%s\", shape=box];" % (i, net[1][i][0]))
        i = i + 1
    i = 0
    while i < len(net[1]):
        for p in net[1][i][1]:
            print("p%d -> t%d;" % (p, i))
        for p in net[1][i][2]:
            print("t%d -> p%d;" % (i, p))
        i = i + 1
    print("}")


def main():
    p = [("source", "p1", "sink"), (("A", {0: 1}, {1: 1}), ("B", {1: 1}, {2: 1}))]
    print_dot(p)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



from micropm4py.petrinet import petrinet_import


# Supports only visible transitions, and with arc weight equal to 1. Supports duplicate transitions.
def imp_file(file_path):
    n = [[], []]
    im = {}
    fm = {}
    st = [False, False, {}, False, None]
    F = open(file_path, "r")
    line = F.readline()
    while line:
        petrinet_import.ip(n, im, fm, st, line)
        line = F.readline()
    F.close()
    petrinet_import.finish_net(n, st)
    return n, im, fm



def st_exp(F):
    F.write("<?xml version='1.0' encoding='UTF-8'?>\n")
    F.write("<log>\n")


def exp_tr_cid(F, tr, cid):
    F.write("\t<trace>\n")
    F.write("\t\t<string key=\"concept:name\" value=\"%s\" />\n" % (cid))
    for act in tr:
        F.write("\t\t<event>\n")
        F.write("\t\t\t<string key=\"concept:name\" value=\"%s\" />\n" % (act))
        F.write("\t\t</event>\n")
    F.write("\t</trace>\n")


def exp_tr(F, tr):
    F.write("\t<trace>\n")
    for act in tr:
        F.write("\t\t<event>\n")
        F.write("\t\t\t<string key=\"concept:name\" value=\"%s\" />\n" % (act))
        F.write("\t\t</event>\n")
    F.write("\t</trace>\n")


def end_exp(F):
    F.write("</log>\n")


def export_traces(list_traces, file_path):
    F = open(file_path, "w")
    st_exp(F)
    for tr in list_traces:
        if len(tr) == 2 and type(tr[1]) is tuple:
            if len(tr[0]) > 0:
                exp_tr_cid(F, tr[1], tr[0])
            else:
                exp_tr(F, tr[1])
        else:
            exp_tr(F, tr)
    end_exp(F)
    F.close()



import time

# RAN ON CORTEX M3, 64kb RAM
def st_exp():
    print("<?xml version='1.0' encoding='UTF-8'?>")
    print("<log>")


def exp_tr_cid(tr, cid):
    print("\t<trace>")
    print("\t\t<string key=\"concept:name\" value=\"%s\" />" % (cid))
    for act in tr:
        print("\t\t<event>")
        print("\t\t\t<string key=\"concept:name\" value=\"%s\" />" % (act))
        print("\t\t</event>")
    print("\t</trace>")


def exp_tr(tr):
    print("\t<trace>")
    for act in tr:
        print("\t\t<event>")
        print("\t\t\t<string key=\"concept:name\" value=\"%s\" />" % (act))
        print("\t\t</event>")
    print("\t</trace>")


def end_exp():
    print("</log>")


def main():
    st_exp()
    exp_tr_cid(["A", "B", "C"], "case1")
    exp_tr(["A", "B"])
    end_exp()


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



# Supports only visible transitions, and with arc weight equal to 1. Supports duplicate transitions.
def export(net, im, fm, file_path):
    F = open(file_path, "w")
    F.write("<?xml version='1.0' encoding='UTF-8'?>\n")
    F.write("<pnml>\n")
    F.write("<net id=\"net1\" type=\"http://www.pnml.org/version-2009/grammar/pnmlcoremodel\">\n")
    F.write("<page id=\"n0\">\n")
    i = 0
    while i < len(net[0]):
        F.write("<place id=\"pl%d\">\n" % (i))
        F.write("<name>\n")
        F.write("<text>%s</text>\n" % (net[0][i]))
        F.write("</name>\n")
        if i in im:
            F.write("<initialMarking>\n")
            F.write("<text>%d</text>\n" % (im[i]))
            F.write("</initialMarking>\n")
        F.write("</place>\n")
        i = i + 1
    a = 0
    i = 0
    while i < len(net[1]):
        F.write("<transition id=\"tr" + str(i) + "\">\n")
        F.write("<name>\n")
        F.write("<text>" + net[1][i][0] + "</text>\n")
        F.write("</name>\n")
        F.write("</transition>\n")
        for p in net[1][i][1]:
            a = a + 1
            F.write("<arc id=\"ac%d\" source=\"pl%d\" target=\"tr%d\"/>\n" % (a, p, i))
        for p in net[1][i][2]:
            a = a + 1
            F.write("<arc id=\"ac%d\" source=\"tr%d\" target=\"pl%d\"/>\n" % (a, i, p))
        i = i + 1
    F.write("</page>\n")
    F.write("<finalmarkings>\n")
    F.write("<marking>\n")
    for p in fm:
        F.write("<place idref=\"pl%d\">\n" % (p))
        F.write("<text>%d</text>\n" % (fm[p]))
        F.write("</place>\n")
    F.write("</marking>\n")
    F.write("</finalmarkings>\n")
    F.write("</net>\n")
    F.write("</pnml>\n")
    F.close()



from micropm4py import log, petrinet, util, conversion, visualization

__version__ = '0.0.8'
__doc__ = "MicroPM4Py - Process Mining for Micro-Controllers"
__author__ = 'Alessandro Berti'
__author_email__ = 'alessandro.berti89@gmail.com'
__maintainer__ = 'Alessandro Berti'
__maintainer_email__ = "alessandro.berti89@gmail.com"



import pm4py.objects.log.log as pm4py_log


def to_trace(list_act, cid):
    t = pm4py_log.Trace()
    t.attributes["concept:name"] = cid
    for act in list_act:
        t.append(pm4py_log.Event({"concept:name": act}))
    return t


def to(trs):
    l = pm4py_log.EventLog()
    i = 0
    while i < len(trs):
        if len(trs[i]) == 2 and type(trs[i][1]) is tuple:
            if len(trs[i][0]) > 0:
                t = to_trace(trs[i][1], trs[i][0])
            else:
                t = to_trace(trs[i][1], str(i))
        else:
            t = to_trace(trs[i], str(i))
        l.append(t)
        i = i + 1
    return l


def frm(log):
    trs = []
    i = 0
    while i < len(log):
        if "concept:name" in log[i].attributes:
            cid = log[i].attributes["concept:name"]
        else:
            cid = str(i)
        list_act = []
        for eve in log[i]:
            list_act.append(eve["concept:name"])
        trs.append((cid, tuple(list_act)))
        i = i + 1
    return trs



import time

# RAN ON CORTEX M3, 64kb RAM
# Supports only visible transitions, and with arc weight equal to 1. Supports duplicate transitions.
def export(net, im, fm):
    print("<?xml version='1.0' encoding='UTF-8'?>")
    print("<pnml>")
    print("<net id=\"net1\" type=\"http://www.pnml.org/version-2009/grammar/pnmlcoremodel\">")
    print("<page id=\"n0\">")
    i = 0
    while i < len(net[0]):
        print("<place id=\"pl%d\">" % (i))
        print("<name>")
        print("<text>%s</text>" % (net[0][i]))
        print("</name>")
        if i in im:
            print("<initialMarking>")
            print("<text>%d</text>" % (im[i]))
            print("</initialMarking>")
        print("</place>")
        i = i + 1
    a = 0
    i = 0
    while i < len(net[1]):
        print("<transition id=\"tr%d\">" % (i))
        print("<name>")
        print("<text>%s</text>" % (net[1][i][0]))
        print("</name>")
        print("</transition>")
        for p in net[1][i][1]:
            a = a + 1
            print("<arc id=\"ac%d\" source=\"pl%d\" target=\"tr%d\"/>" % (a, p, i))
        for p in net[1][i][2]:
            a = a + 1
            print("<arc id=\"ac%d\" source=\"tr%d\" target=\"pl%d\"/>" % (a, i, p))
        i = i + 1
    print("</page>")
    print("<finalmarkings>")
    print("<marking>")
    for p in fm:
        print("<place idref=\"pl%d\">" % (p))
        print("<text>%d</text>" % (fm[p]))
        print("</place>")
    print("</marking>")
    print("</finalmarkings>")
    print("</net>")
    print("</pnml>")


def main():
    p = [("source", "p1", "sink"), (("A", {0: 1}, {1: 1}), ("B", {1: 1}, {2: 1}))]
    export(p, {0: 1}, {2: 1})


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



def save_dot(file_path, dfg):
    F = open(file_path, "w")
    F.write("digraph graphname {\n")
    i = 0
    while i < len(dfg[0]):
        F.write("%d [label=\"%s (%d)\"];\n" % (i, dfg[0][i], dfg[1][i]))
        i = i + 1
    for tup in dfg[4]:
        F.write("%d -> %d [label=\"%d\"];\n" % (tup[0], tup[1], dfg[4][tup]))
    F.write("}\n")
    F.close()



def add_art_st_end(log):
    i = 0
    while i < len(log):
        if len(log[i]) == 2 and type(log[i][1]) is tuple:
            log[i] = list(log[i])
            log[i][1] = [">>"] + list(log[i][1]) + ["[]"]
            log[i][1] = tuple(log[i][1])
            log[i] = tuple(log[i])
        i = i + 1
    return log



def exp_tr_cid(F, tr, sep, ci):
    for act in tr:
        F.write("%s%s%s\n" % (ci, sep, act))

def exp_hea(F, sep, ci, ai):
    F.write("%s%s%s\n" % (ci, sep, ai))

def exp_trs(F, trs, sep, ci, ai):
    exp_hea(F, sep, ci, ai)
    i = 0
    while i < len(trs):
        if len(trs[i]) == 2 and type(trs[i][1]) is tuple:
            if len(trs[i][0]) > 0:
                exp_tr_cid(F, trs[i][1], ",", trs[i][0])
            else:
                exp_tr_cid(F, trs[i][1], ",", str(i))
        else:
            exp_tr_cid(F, trs[i], ",", str(i))
        i = i + 1

def export_to_path(trs, path, sep, ci, ai):
    F = open(path, "w")
    exp_trs(F, trs, sep, ci, ai)
    F.close()



from micropm4py.log import xes_import_traces


def imp_list_traces_from_file(file_path):
    list_traces = []
    tr = None
    p = None
    vv = {}
    on = 0
    d = {}
    F = open(file_path, "r")
    line = F.readline()
    while line:
        tr, p, on = xes_import_traces.r(tr, p, on, line, d)
        if tr is not None:
            if tr[1] not in vv:
                vv[tr[1]] = tr[1]
            tr[1] = vv[tr[1]]
            list_traces.append(tuple(tr))
        line = F.readline()
    F.close()
    return list_traces


def imp_dfg_file(file_path):
    dfg = [[], dict(), set(), set(), dict()]
    tr = None
    p = None
    on = 0
    d = {}
    F = open(file_path, "r")
    line = F.readline()
    while line:
        tr, p, on = xes_import_traces.r(tr, p, on, line, d)
        if tr is not None:
            dfg = xes_import_traces.udf(dfg, tr[1])
        line = F.readline()
    F.close()
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    return dfg


def imp_dfg_file_sten(file_path):
    dfg = [[], dict(), set(), set(), dict()]
    tr = None
    p = None
    on = 0
    d = {}
    F = open(file_path, "r")
    line = F.readline()
    while line:
        tr, p, on = xes_import_traces.r(tr, p, on, line, d)
        if tr is not None:
            dfg = xes_import_traces.udf_sten(dfg, tr[1])
        line = F.readline()
    F.close()
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    return dfg



import time

# RAN ON CORTEX M3, 64kb RAM
def apply(dfg):
    n = [[], []]
    im = {}
    fm = {}
    if len(dfg[2]) > 1 or len(dfg[3]) > 1:
        raise Exception("only DFGs with a single start activity/end activity are now supported! Try to add artificial start/end activities")
    for act in dfg[0]:
        n[0].append(act)
    for tup in dfg[4]:
        n[1].append([dfg[0][tup[1]], {tup[0]: 1}, {tup[1]: 1}])
    n[0].append("@@source@@")
    n[1].append((dfg[0][list(dfg[2])[0]], {len(n[0])-1:1}, {list(dfg[2])[0]:1}))
    n[0] = tuple(n[0])
    n[1] = tuple(n[1])
    im[len(n[0])-1] = 1
    fm[list(dfg[3])[0]] = 1
    return n, im, fm


def main():
    dfg = [("A", "B", "C"), {0:1, 1:1, 2:1}, tuple({0}), tuple({2}), {(0, 1): 1, (1, 2): 1}]
    n, im, fm = apply(dfg)
    print(n)
    print(im)
    print(fm)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



from os.path import dirname, join

from setuptools import setup

import micropm4py


def read_file(filename):
    with open(join(dirname(__file__), filename)) as f:
        return f.read()


setup(
    name=micropm4py.__name__,
    version=micropm4py.__version__,
    description=micropm4py.__doc__.strip(),
    long_description=read_file('README.md'),
    author=micropm4py.__author__,
    author_email=micropm4py.__author_email__,
    py_modules=[micropm4py.__name__],
    include_package_data=True,
    packages=['micropm4py', 'micropm4py.log', 'micropm4py.util', 'micropm4py.petrinet',
              'micropm4py.conversion', 'micropm4py.conversion.dfg', 'micropm4py.conversion.pm4py', 'micropm4py.visualization'],
    url='http://www.pm4py.org',
    license='GPL 3.0',
    install_requires=[
    ],
    project_urls={
        'Documentation': 'https://www.micropm4py.org/documentation.html',
        'Source': 'https://github.com/Javert899/micropm4py/tree/master/micropm4py',
        'Tracker': 'https://github.com/Javert899/micropm4py/issues',
    }
)



import time

# RAN ON CORTEX M3, 64kb RAM
def import_header(l, sep, ci, ai):
    l = l.split(sep)
    cidp = -1
    acp = -1
    i = 0
    while i < len(l):
        if l[i] == ci:
            cidp = i
        elif l[i] == ai:
            acp = i
        i = i + 1
    return cidp, acp


def import_traces(d, l, sep, cidp, acp, da):
    l = l.rstrip().split(sep)
    if l[cidp] not in d:
        d[l[cidp]] = []
    a = l[acp]
    if a not in da:
        da[a] = a
    d[l[cidp]].append(da[a])
    return d


def finish_traces(d):
    r = []
    vv = {}
    kk = list(d.keys())
    while kk:
        d[kk[0]] = tuple(d[kk[0]])
        if d[kk[0]] not in vv:
            vv[d[kk[0]]] = d[kk[0]]
        r.append((kk[0], vv[d[kk[0]]]))
        del d[kk[0]]
        del kk[0]
    return r


def import_dfg(dfg, d, l, sep, cidp, acp):
    l = l.rstrip().split(sep)
    a = l[acp]
    if a not in dfg[0]:
        dfg[0].append(a)
        dfg[1][len(dfg[0])-1] = 0
    dfg[1][dfg[0].index(a)] = dfg[1][dfg[0].index(a)] + 1
    if l[cidp] not in d:
        dfg[2].add(dfg[0].index(a))
    else:
        tup = (dfg[0].index(d[l[cidp]]), dfg[0].index(a))
        if tup not in dfg[4]:
            dfg[4][tup] = 0
        dfg[4][tup] = dfg[4][tup] + 1
    d[l[cidp]] = a
    return dfg, d


def import_dfg_sten(dfg, d, l, sep, cidp, acp):
    l = l.rstrip().split(sep)
    a = l[acp]
    if a not in dfg[0]:
        dfg[0].append(a)
        dfg[1][len(dfg[0])-1] = 0
    dfg[1][dfg[0].index(a)] = dfg[1][dfg[0].index(a)] + 1
    if l[cidp] not in d:
        d[l[cidp]] = ">>"
        dfg[1][0] = dfg[1][0] + 1
    tup = (dfg[0].index(d[l[cidp]]), dfg[0].index(a))
    if tup not in dfg[4]:
        dfg[4][tup] = 0
    dfg[4][tup] = dfg[4][tup] + 1
    d[l[cidp]] = a
    return dfg, d


def finish_dfg(dfg, d):
    for c in d:
        dfg[3].add(dfg[0].index(d[c]))
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    return dfg


def finish_dfg_sten(dfg, d):
    for c in d:
        tup = (dfg[0].index(d[c]), dfg[0].index("[]"))
        if tup not in dfg[4]:
            dfg[4][tup] = 0
        dfg[4][tup] = dfg[4][tup] + 1
        dfg[1][1] = dfg[1][1] + 1
    return dfg


def main():
    cidp, acp = import_header("case:concept:name,concept:name", ",", "case:concept:name", "concept:name")
    d = {}
    da = {}
    d = import_traces(d, "1,A", ",", cidp, acp, da)
    d = import_traces(d, "2,A", ",", cidp, acp, da)
    d = import_traces(d, "1,B", ",", cidp, acp, da)
    d = import_traces(d, "2,B", ",", cidp, acp, da)
    d = import_traces(d, "1,C", ",", cidp, acp, da)
    d = finish_traces(d)
    print(d)


def main2():
    cidp, acp = import_header("case:concept:name,concept:name", ",", "case:concept:name", "concept:name")
    dfg = [[], dict(), set(), set(), dict()]
    d = {}
    dfg, d = import_dfg(dfg, d, "1,A", ",", cidp, acp)
    dfg, d = import_dfg(dfg, d, "2,A", ",", cidp, acp)
    dfg, d = import_dfg(dfg, d, "1,B", ",", cidp, acp)
    dfg, d = import_dfg(dfg, d, "2,B", ",", cidp, acp)
    dfg, d = import_dfg(dfg, d, "1,C", ",", cidp, acp)
    dfg = finish_dfg(dfg, d)
    print(dfg)


def main3():
    cidp, acp = import_header("case:concept:name,concept:name", ",", "case:concept:name", "concept:name")
    dfg = [[">>", "[]"], {0: 0, 1: 0}, {0}, {1}, dict()]
    d = {}
    dfg, d = import_dfg_sten(dfg, d, "1,A", ",", cidp, acp)
    dfg, d = import_dfg_sten(dfg, d, "2,A", ",", cidp, acp)
    dfg, d = import_dfg_sten(dfg, d, "1,B", ",", cidp, acp)
    dfg, d = import_dfg_sten(dfg, d, "2,B", ",", cidp, acp)
    dfg, d = import_dfg_sten(dfg, d, "1,C", ",", cidp, acp)
    dfg = finish_dfg_sten(dfg, d)
    print(dfg)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main3()
    bb = time.ticks_ms()
    print(bb-aa)



import time

# RAN ON CORTEX M3, 64kb RAM

def exp_tr_cid(tr, sep, ci):
    for act in tr:
        print("%s%s%s" % (ci, sep, act))

def exp_hea(sep, ci, ai):
    print("%s%s%s" % (ci, sep, ai))

def exp_trs(trs, sep, ci, ai):
    exp_hea(sep, ci, ai)
    i = 0
    while i < len(trs):
        if len(trs[i]) == 2 and type(trs[i][1]) is tuple:
            if len(trs[i][0]) > 0:
                exp_tr_cid(trs[i][1], ",", trs[i][0])
            else:
                exp_tr_cid(trs[i][1], ",", str(i))
        else:
            exp_tr_cid(trs[i], ",", str(i))
        i = i + 1

def main():
    exp_hea(",", "case:concept:name", "concept:name")
    exp_tr_cid(("A", "B", "C"), ",", "c1")
    exp_tr_cid(("A", "B"), ",", "c2")


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



import time

# RAN ON CORTEX M3, 64kb RAM
def r(tr, p, on, row, d):
    if on == 2:
        if "concept:name" in row:
            a = row.split("\"")[3]
            if a not in d:
                d[a] = a
            p[1].append(d[a])
        elif "</event" in row:
            on = 1
    elif on == 1:
        if "<event" in row:
            on = 2
        elif "</trace" in row:
            p[1] = tuple(p[1])
            tr = p
            p = None
            on = 0
        elif "concept:name" in row:
            p[0] = row.split("\"")[3]
    else:
        tr = None
        if "<trace" in row:
            p = ["", []]
            on = 1
    return tr, p, on


def udf(dfg, tr):
    if len(tr) > 0:
        i = 0
        while i < len(tr):
            if not tr[i] in dfg[0]:
                dfg[0].append(tr[i])
                dfg[1][len(dfg[0]) - 1] = 0
            dfg[1][dfg[0].index(tr[i])] = dfg[1][dfg[0].index(tr[i])] + 1
            if i > 0:
                tup = (dfg[0].index(tr[i - 1]), dfg[0].index(tr[i]))
                if tup not in dfg[4]:
                    dfg[4][tup] = 0
                dfg[4][tup] = dfg[4][tup] + 1
            i = i + 1
        dfg[2].add(dfg[0].index(tr[0]))
        dfg[3].add(dfg[0].index(tr[-1]))
    return dfg


def udf_sten(dfg, tr):
    if len(tr) > 0:
        tr = [">>"] + list(tr) + ["[]"]
        i = 0
        while i < len(tr):
            if not tr[i] in dfg[0]:
                dfg[0].append(tr[i])
                dfg[1][len(dfg[0]) - 1] = 0
            dfg[1][dfg[0].index(tr[i])] = dfg[1][dfg[0].index(tr[i])] + 1
            if i > 0:
                tup = (dfg[0].index(tr[i - 1]), dfg[0].index(tr[i]))
                if tup not in dfg[4]:
                    dfg[4][tup] = 0
                dfg[4][tup] = dfg[4][tup] + 1
            i = i + 1
        dfg[2].add(dfg[0].index(tr[0]))
        dfg[3].add(dfg[0].index(tr[-1]))
    return dfg

def main():
    tr = None
    p = None
    on = 0
    dfg = [[], dict(), set(), set(), dict()]
    d = {}
    tr, p, on = r(tr, p, on, "<?xml version='1.0' encoding='UTF-8'?>", d)
    tr, p, on = r(tr, p, on, "<log>", d)
    tr, p, on = r(tr, p, on, "\t<trace>", d)
    tr, p, on = r(tr, p, on, "\t\t<string key=\"concept:name\" value=\"ciao\" />", d)
    tr, p, on = r(tr, p, on, "\t\t<event>", d)
    tr, p, on = r(tr, p, on, "\t\t\t<string key=\"concept:name\" value=\"A\" />", d)
    tr, p, on = r(tr, p, on, "\t\t</event>", d)
    tr, p, on = r(tr, p, on, "\t\t<event>", d)
    tr, p, on = r(tr, p, on, "\t\t\t<string key=\"concept:name\" value=\"B\" />", d)
    tr, p, on = r(tr, p, on, "\t\t</event>", d)
    tr, p, on = r(tr, p, on, "\t\t<event>", d)
    tr, p, on = r(tr, p, on, "\t\t\t<string key=\"concept:name\" value=\"C\" />", d)
    tr, p, on = r(tr, p, on, "\t\t</event>", d)
    tr, p, on = r(tr, p, on, "\t</trace>", d)
    print(tr)
    dfg = udf_sten(dfg, tr[1])
    tr, p, on = r(tr, p, on, "\t<trace>", d)
    tr, p, on = r(tr, p, on, "\t\t<event>", d)
    tr, p, on = r(tr, p, on, "\t\t\t<string key=\"concept:name\" value=\"A\" />", d)
    tr, p, on = r(tr, p, on, "\t\t</event>", d)
    tr, p, on = r(tr, p, on, "\t\t<event>", d)
    tr, p, on = r(tr, p, on, "\t\t\t<string key=\"concept:name\" value=\"B\" />", d)
    tr, p, on = r(tr, p, on, "\t\t</event>", d)
    tr, p, on = r(tr, p, on, "\t</trace>", d)
    print(tr)
    dfg = udf_sten(dfg, tr[1])
    dfg[0] = tuple(dfg[0])
    dfg[2] = tuple(dfg[2])
    dfg[3] = tuple(dfg[3])
    tr, p, on = r(tr, p, on, "</log>", d)
    print(dfg)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)



import time

# RAN ON CORTEX M3, 64kb RAM
# Supports only visible transitions, and with arc weight equal to 1. Supports duplicate transitions.
def ip(n, im, fm, st, row):
    if "</transition" in row:
        st[0] = False
    elif "</initialMark" in row:
        st[3] = False
    elif "</finalmark" in row:
        st[1] = False
    if st[0]:
        if "<text" in row:
            st[2][n[1][-1][0]] = row.split("<text>")[1].split("</")[0]
    elif st[3]:
        if "<text" in row:
            im[len(n[0]) - 1] = int(row.split("<text>")[1].split("</")[0])
    elif st[1]:
        if "<place" in row:
            st[4] = n[0].index(row.split("idref=\"")[1].split("\"")[0])
        elif "<text" in row:
            fm[st[4]] = int(row.split("<text>")[1].split("</")[0])
    elif "<place" in row:
        n[0].append(row.split("\"")[1])
    elif "<arc" in row:
        rs = row.split("\"")
        so = rs[3]
        ta = rs[5]
        if so in n[0]:
            i = 0
            while i < len(n[1]):
                if n[1][i][0] == ta:
                    break
                i = i + 1
            n[1][i][1][n[0].index(so)] = 1
        else:
            i = 0
            while i < len(n[1]):
                if n[1][i][0] == so:
                    break
                i = i + 1
            n[1][i][2][n[0].index(ta)] = 1
    elif "<transition" in row:
        st[0] = True
        n[1].append([row.split("id=\"")[1].split("\"")[0], {}, {}])
    elif "<initialMark" in row:
        st[3] = True
    elif "<finalmark" in row:
        st[1] = True


def finish_net(n, st):
    n[0] = tuple(n[0])
    i = 0
    while i < len(n[1]):
        n[1][i][0] = st[2][n[1][i][0]]
        n[1][i] = tuple(n[1][i])
        i = i + 1
    n[1] = tuple(n[1])


def main():
    n = [[], []]
    im = {}
    fm = {}
    st = [False, False, {}, False, None]
    ip(n, im, fm, st, "<?xml version='1.0' encoding='UTF-8'?>")
    ip(n, im, fm, st, "<pnml>")
    ip(n, im, fm, st, "<net id=\"net1\" type=\"http://www.pnml.org/version-2009/grammar/pnmlcoremodel\">")
    ip(n, im, fm, st, "<page id=\"n0\">")
    ip(n, im, fm, st, "<place id=\"source\">")
    ip(n, im, fm, st, "<name>")
    ip(n, im, fm, st, "<text>source</text>")
    ip(n, im, fm, st, "</name>")
    ip(n, im, fm, st, "<initialMarking>")
    ip(n, im, fm, st, "<text>1</text>")
    ip(n, im, fm, st, "</initialMarking>")
    ip(n, im, fm, st, "</place>")
    ip(n, im, fm, st, "<place id=\"p1\">")
    ip(n, im, fm, st, "<name>")
    ip(n, im, fm, st, "<text>p1</text>")
    ip(n, im, fm, st, "</name>")
    ip(n, im, fm, st, "</place>")
    ip(n, im, fm, st, "<place id=\"sink\">")
    ip(n, im, fm, st, "<name>")
    ip(n, im, fm, st, "<text>sink</text>")
    ip(n, im, fm, st, "</name>")
    ip(n, im, fm, st, "</place>")
    ip(n, im, fm, st, "<transition id=\"A\">")
    ip(n, im, fm, st, "<name>")
    ip(n, im, fm, st, "<text>A</text>")
    ip(n, im, fm, st, "</name>")
    ip(n, im, fm, st, "</transition>")
    ip(n, im, fm, st, "<arc id=\"ac1\" source=\"source\" target=\"A\"/>")
    ip(n, im, fm, st, "<arc id=\"ac2\" source=\"A\" target=\"p1\"/>")
    ip(n, im, fm, st, "<transition id=\"B\">")
    ip(n, im, fm, st, "<name>")
    ip(n, im, fm, st, "<text>B</text>")
    ip(n, im, fm, st, "</name>")
    ip(n, im, fm, st, "</transition>")
    ip(n, im, fm, st, "<arc id=\"ac3\" source=\"p1\" target=\"B\"/>")
    ip(n, im, fm, st, "<arc id=\"ac4\" source=\"B\" target=\"sink\"/>")
    ip(n, im, fm, st, "</page>")
    ip(n, im, fm, st, "<finalmarkings>")
    ip(n, im, fm, st, "<marking>")
    ip(n, im, fm, st, "<place idref=\"sink\">")
    ip(n, im, fm, st, "<text>1</text>")
    ip(n, im, fm, st, "</place>")
    ip(n, im, fm, st, "</marking>")
    ip(n, im, fm, st, "</finalmarkings>")
    ip(n, im, fm, st, "</net>")
    ip(n, im, fm, st, "</pnml>")
    finish_net(n, st)
    print(n)
    print(st)
    print(im)
    print(fm)


if __name__ == "__main__":
    aa = time.ticks_ms()
    main()
    bb = time.ticks_ms()
    print(bb-aa)
